@@ -14,7 +14,9 @@ setMethod("pathInc", signature(transition = "TransitionLayer", origin = "Coords"
 	{
 		origin <- .coordsToMatrix(origin)
 		from <- .coordsToMatrix(fromCoords)
-		prepared <- .preparationFlow(transition, origin, fromCoords, type, 0)
+		preparedMatrix <- .preparationMatrix(transition, 0)
+		preparedIndex <- .preparationIndex1(transition, origin, from, type)
+		prepared <- c(preparedMatrix,preparedIndex,list(type=type))
 		Intermediate <- .randomWalk(prepared)
 		result <- .finishFlow(prepared, Intermediate)
 		return(result)
@@ -28,7 +30,9 @@ setMethod("pathInc", signature(transition = "TransitionLayer", origin = "Coords"
 		if(theta < 0 | theta > 20 ) {stop("theta value out of range (between 0 and 20)")}
 		origin <- .coordsToMatrix(origin)
 		from <- .coordsToMatrix(fromCoords)
-		prepared <- .preparationFlow(transition, origin, fromCoords, type, 0)
+		preparedMatrix <- .preparationMatrix(transition, 0)
+		preparedIndex <- .preparationIndex1(transition, origin, from, type)
+		prepared <- c(preparedMatrix,preparedIndex,list(type=type))
 		Intermediate <- .randomSP(prepared, theta)
 		result <- .finishFlow(prepared, Intermediate)
 		return(result)
@@ -41,7 +45,9 @@ setMethod("pathInc", signature(transition = "TransitionLayer", origin = "Coords"
 	{
 		origin <- .coordsToMatrix(origin)
 		from <- .coordsToMatrix(fromCoords)
-		prepared <- .preparationFlow(transition, origin, fromCoords, type, weight)
+		preparedMatrix <- .preparationMatrix(transition, weight)
+		preparedIndex <- .preparationIndex1(transition, origin, from, type)
+		prepared <- c(preparedMatrix,preparedIndex,list(type=type))
 		Intermediate <- .randomWalk(prepared)
 		result <- .finishFlow(prepared, Intermediate)
 		return(result)
@@ -55,7 +61,9 @@ setMethod("pathInc", signature(transition = "TransitionLayer", origin = "Coords"
 		if(theta < 0 | theta > 20 ) {stop("theta value out of range (between 0 and 20)")}
 		origin <- .coordsToMatrix(origin)
 		from <- .coordsToMatrix(fromCoords)
-		prepared <- .preparationFlow(transition, origin, fromCoords, type, weight)
+		preparedMatrix <- .preparationMatrix(transition, weight)
+		preparedIndex <- .preparationIndex1(transition, origin, from, type)
+		prepared <- c(preparedMatrix,preparedIndex,list(type=type))
 		Intermediate <- .randomSP(prepared, theta)
 		result <- .finishFlow(prepared, Intermediate)
 		return(result)
@@ -68,7 +76,9 @@ setMethod("pathInc", signature(transition = "TransitionLayer", origin = "Coords"
 	{
 		origin <- .coordsToMatrix(origin)
 		from <- .coordsToMatrix(fromCoords)
-		prepared <- .preparationFlow(transition, origin, fromCoords, type, weight)
+		preparedMatrix <- .preparationMatrix(transition, weight)
+		preparedIndex <- .preparationIndex1(transition, origin, from, type)
+		prepared <- c(preparedMatrix,preparedIndex,list(type=type))
 		Intermediate <- .randomWalk(prepared)
 		result <- .finishFlowStack(prepared, Intermediate)
 		return(result)
@@ -82,21 +92,21 @@ setMethod("pathInc", signature(transition = "TransitionLayer", origin = "Coords"
 		if(theta < 0 | theta > 20 ) {stop("theta value out of range (between 0 and 20)")}
 		origin <- .coordsToMatrix(origin)
 		from <- .coordsToMatrix(fromCoords)
-		prepared <- .preparationFlow(transition, origin, fromCoords, type, weight)
+		preparedMatrix <- .preparationMatrix(transition, weight)
+		preparedIndex <- .preparationIndex1(transition, origin, from, type)		
+		prepared <- c(preparedMatrix,preparedIndex,list(type=type))
 		Intermediate <- .randomSP(prepared, theta)
 		result <- .finishFlowStack(prepared, Intermediate)
 		return(result)
 	}
 )
 
-.preparationFlow <- function(transition, origin, fromCoords, type, weight)
+.preparationIndex1 <- function(transition, origin, fromCoords, type)
 {
 		if(!all(type %in% c("divergent","joint"))) {stop("type can only have values \'joint\' and/or \'divergent\'")}
 
 		originCell <- cellFromXY(transition, origin)
 
-		transition <- .transitionSolidify(transition)
-
 		if (!(originCell %in% transitionCells(transition))) {stop("the origin refers to a zero row/column in the transition matrix (unconnected)")} 
 
 		allFromCells <- cellFromXY(transition, fromCoords)
@@ -109,13 +119,30 @@ setMethod("pathInc", signature(transition = "TransitionLayer", origin = "Coords"
 
 		indexCoords <- match(fromCells,transitionCells(transition))
 		indexOrigin <- match(originCell,transitionCells(transition))
+		
 
+		
+		result <- list(transition=transition,
+				fromCoords=fromCoords,
+				allFromCells=allFromCells, 
+				fromCells=fromCells,
+				indexCoords=indexCoords, 
+				indexOrigin=indexOrigin)
+		return(result)
+}
+		
+.preparationMatrix <- function(transition, weight)
+{
+		
+		transition <- .transitionSolidify(transition)
+		
 		A <- as(transitionMatrix(transition),"lMatrix")
 		A <- as(A,"dMatrix")
 		AIndex <- as(A, "dgTMatrix")
 		index1 <- cbind(transitionCells(transition)[as.integer(AIndex@i+1)],transitionCells(transition)[as.integer(AIndex@j+1)]) 
 		index2 <- cbind(as.integer(AIndex@i+1),as.integer(AIndex@j+1))
 		#if symmetric? index <- index[index[,1] < index[,2],]
+		
 		Size <- nrow(index1)
 		
 		if(class(weight) == "numeric")
@@ -125,7 +152,7 @@ setMethod("pathInc", signature(transition = "TransitionLayer", origin = "Coords"
 		}
 		if(class(weight) == "TransitionLayer")
 		{
-			R <- 1/weight[index1]
+			if(matrixValues(weight) == "conductance"){R <- 1/weight[index1]} else{R <- weight[index1]} 
 			R[R == Inf] <- 0
 		}
 		if(class(weight) == "TransitionStack")
@@ -133,22 +160,16 @@ setMethod("pathInc", signature(transition = "TransitionLayer", origin = "Coords"
 			R <- matrix(nrow=nlayers(weight), ncol=length(index1[,1]))
 			for(i in 1:nlayers(weight))
 			{
-				R[i,] <- 1/weight[[i]][index1] 
+				if(matrixValues(weight[[i]]) == "conductance"){R[i,] <- 1/weight[[i]][index1]} else{R[i,] <- weight[[i]][index1]}
 			}
 			R[R == Inf] <- 0
 		}
 
 		result <- list(transition=transition,
-						type=type,
-						fromCoords=fromCoords,
-						allFromCells=allFromCells, 
-						fromCells=fromCells,
-						indexCoords=indexCoords, 
-						indexOrigin=indexOrigin,
 						index=index2,
-						Size=Size,
 						A=A,
-						R=R)
+						R=R,
+						Size=Size)
 		return(result)
 }
 
@@ -168,7 +189,7 @@ setMethod("pathInc", signature(transition = "TransitionLayer", origin = "Coords"
 	n <- max(Lr@Dim)
 	Lr <- Cholesky(Lr)
 
-	if(!(canProcessInMemory(transition, length(fromCells)))) #depending on memory availability, currents are calculated in a piecemeal fashion or all at once
+	if(!(canProcessInMemory(transition, length(fromCells)*10))) #depending on memory availability, currents are calculated in a piecemeal fashion or all at once
 	{
 		filenm=rasterTmpFile()
 		Flow <- raster(nrows=length(fromCells), ncols=Size)
@@ -217,7 +238,7 @@ setMethod("pathInc", signature(transition = "TransitionLayer", origin = "Coords"
 	W@x <- exp(-theta * trR@x) #zero values are not relevant because of next step exp(-theta * trR@x)
 	W <- W * P 
 
-	if(!(canProcessInMemory(transition, length(fromCells)))) 
+	if(!(canProcessInMemory(transition, length(fromCells)*10))) 
 	#this does not take into account the exact memory needed for matrix solving...
 	{
 		filenm=rasterTmpFile()
@@ -489,33 +510,41 @@ setMethod("pathInc", signature(transition = "TransitionLayer", origin = "Coords"
 
 	if("divergent" %in% type)
 	{
+		divFlnew <- matrix(nrow=length(allFromCells),ncol=length(allFromCells))
+		rownames(divFlnew) <- rownames(fromCoords)
+		colnames(divFlnew) <- rownames(fromCoords)
+		
 		for(i in 1:nrow(R))
 		{
-			divFl <- matrix(nrow=length(allFromCells),ncol=length(allFromCells))
-			rownames(divFl) <- rownames(fromCoords)
-			colnames(divFl) <- rownames(fromCoords)
+			divFl <- divFlnew
 			divFlowi <- as.matrix(as.dist(divFlow[[i]], diag=TRUE))
 			divFl[index1,index1] <- divFlowi[index2,index2]
 			divFl <- as.dist(divFl)
-			attr(divFl, "method") <- "divergent path"
+			divFl <- as.vector(divFl)
 			divFlow[[i]] <- divFl
 		}
+		divFlow <- as.data.frame(divFlow)
+		colnames(divFlow) <- paste("div", 1:nrow(R), sep="")
 	}
 	if("joint" %in% type)
 	{
+		jointFlnew <- matrix(nrow=length(allFromCells),ncol=length(allFromCells))
+		rownames(jointFlnew) <- rownames(fromCoords)
+		colnames(jointFlnew) <- rownames(fromCoords)
+		
 		for(i in 1:nrow(R))
 		{
-			jointFl <- matrix(nrow=length(allFromCells),ncol=length(allFromCells))
-			rownames(jointFl) <- rownames(fromCoords)
-			colnames(jointFl) <- rownames(fromCoords)
+			jointFl <- jointFlnew
 			jointFlowi <- as.matrix(as.dist(jointFlow[[i]], diag=TRUE))
 			jointFl[index1,index1] <- jointFlowi[index2,index2]
 			jointFl <- as.dist(jointFl)
-			attr(jointFl, "method") <- "joint path"	
+			jointFl <- as.vector(jointFl)
 			jointFlow[[i]] <- jointFl
 		}
+		jointFlow <- as.data.frame(jointFlow)
+		colnames(jointFlow) <- paste("joint", 1:nrow(R), sep="")
 	}
-	if(length(type) > 1) {return(list(divergent=divFlow, joint=jointFlow))}
+	if(length(type) > 1) {return(cbind(divFlow, jointFlow))}
 	if(length(type) == 1 & type == "divergent") {return(divFlow)}
 	if(length(type) == 1 & type == "joint") {return(jointFlow)}
 }
\ No newline at end of file
